##
## Finding Chart plugin for SkyCat
##
## 13/07/2001
##
## S.Marteau (smarteau@eso.org)
##
## $Id: FindingChart_plugin.tcl,v 1.18 2004/05/05 13:48:05 smarteau Exp $


proc SkyCat_plugin {this} {

    set w [utilNamespaceTail $this]

    add_fc_features $w

}


proc min { a b } {

    if { $a <= $b } {

	return $a

    } else {

	return $b

    }
}

proc add_fc_features {w} {

    global fcPlugVar

    set fcPlugVar(startDir) [pwd]

    set m [$w get_menu File]

    set im [$w component image]

    $w insert_menuitem $m 5 command "Make finding chart..." \
	{Make a finding chart} \
	-command [code fc_makeFCDialog $w] \
	-accelerator {Control-F}

    # Disable flipping and rotating functionalities
    set RtdImage [$w component image]
    set RtdImagePanel [$RtdImage component panel]
    
    pack forget $RtdImagePanel.info.trans.rotate
    pack forget $RtdImagePanel.info.trans.flipx
    pack forget $RtdImagePanel.info.trans.flipy

    # Bindings for center-zoom features
    set canvas [$im component canvas]

    bind $canvas <Shift-Button-3> [code fc_recenter $w %x %y]

    bind $canvas <Alt-Z> +[code fc_zoom_in $w]
    bind $canvas <Alt-z> +[code fc_zoom_out $w]

    # Change behaviour of RTD zoom in/out buttons to reflect the <Alt-Z>
    # and <Alt-z> shortcuts
    .skycat1.image.panel.info.trans.larger configure -command [code fc_zoom_in .skycat1]
    .skycat1.image.panel.info.trans.smaller configure -command [code fc_zoom_out .skycat1]

    set fcPlugVar(jpegResolution) "high-res (mandatory for MOS / masks)"

}


proc fc_makeFCDialog { w } {

    global fcPlugVar

    if { [winfo exists .fcDialog] == 1 } {
        raise .fcDialog
        return
    }
    
    toplevel .fcDialog 

    wm title .fcDialog "Finding Chart information"

    if { [info exists fcPlugVar(runID)] == 0 } {
	set fcPlugVar(runID) ""
    }
    
    if { [info exists fcPlugVar(piName)] == 0 } {
	set fcPlugVar(piName) ""
    }
    
    if { [info exists fcPlugVar(obName)] == 0 } {
	set fcPlugVar(obName) ""
    }

    if { [info exists fcPlugVar(band)] == 0 } {
	set fcPlugVar(band) ""
    }

    set w1 [LabelEntry .fcDialog.runID -text "Run ID:" -orient horizontal \
		-valuewidth 20 -labelwidth 17 \
		-state normal -textvariable fcPlugVar(runID) \
		-value $fcPlugVar(runID)] 

    set w2 [LabelEntry .fcDialog.piName -text "PI Name:" -orient horizontal \
		-valuewidth 20 -labelwidth 17 \
		-state normal -textvariable fcPlugVar(piName) \
		-value $fcPlugVar(piName)]

    set w3 [LabelEntry .fcDialog.obName -text "List of OBs:" -orient horizontal \
		-valuewidth 20 -labelwidth 17 \
		-state normal -textvariable fcPlugVar(obName) \
		-value $fcPlugVar(obName) ]

    set w4 [LabelEntry .fcDialog.band -text "Band / Wavelength\n of this chart:" -orient horizontal \
		-valuewidth 20 -labelwidth 17 \
		-state normal -textvariable fcPlugVar(band) \
		-value $fcPlugVar(band) ]

    pack $w1 $w2 $w3 $w4 -side top -expand yes -fill both -padx 4 -ipady 3


    set w1 [LabelMenu .fcDialog.infoLocation -text "Location of info:" \
		-labelwidth 17 -valuewidth 20 \
		-relief raised -orient horizontal -valuefont 8x13 ]

    set w2 [LabelMenu .fcDialog.infoColor -text "Text color:" \
		-labelwidth 17 -valuewidth 20 \
		-relief raised -orient horizontal -valuefont 8x13 ]

    set w3 [LabelChoice .fcDialog.fcResolution \
		-text {Jpeg size:             } -rows 1 \
		-choice {"normal" "high-res (mandatory for MOS / masks)"} \
		-anchor w -borderwidth 2 -relief groove  -variable fcPlugVar(jpegResolution)]

    [.fcDialog.fcResolution component table] configure -relief flat

    pack $w1 $w2 $w3 -side top -expand yes -fill both -padx 4 -ipady 4

    $w1 add -label "top-left corner"
    $w1 add -label "top-right corner"
    $w1 add -label "bottom-left corner"
    $w1 add -label "bottom-right corner"
    $w1 add_separator
    $w1 add -label "user chosen"

    $w2 add -label "black"
    $w2 add -label "white"



    frame .fcDialog.buttonFrame

    button .fcDialog.buttonFrame.draw -text "Draw" \
        -command [code fc_drawFindingChart $w]

    button .fcDialog.buttonFrame.save -text "Save to jpeg..." \
        -command [code fc_saveFindingChart $w]
    
    button .fcDialog.buttonFrame.print -text "Print..." \
        -command [code fc_printFindingChart $w]
    
    button .fcDialog.buttonFrame.reset -text "Reset coords" \
        -command { set fcPlugVar(userChosenFlag) "false" }

    button .fcDialog.buttonFrame.clear -text "Clear" \
        -command [code fc_clearFindingChart $w]

    button .fcDialog.buttonFrame.dismiss -text "Close" \
        -command { destroy .fcDialog}

    pack .fcDialog.buttonFrame.draw .fcDialog.buttonFrame.save \
	    .fcDialog.buttonFrame.print \
	    .fcDialog.buttonFrame.reset .fcDialog.buttonFrame.clear \
	    .fcDialog.buttonFrame.dismiss \
	    -side left -fill both -padx 5

    pack .fcDialog.buttonFrame -side top -fill both -expand yes -padx 5 -pady 8

    set fcPlugVar(userChosenFlag) "false"
    set fcPlugVar(draw) "false"

    return
}


# Procedure to recenter the image displayed in the Skycat interface,
# based on the current mouse position. See binding in procedure
# "add_fc_features".
proc fc_recenter {w x y} {

    # Retrieve name of underlying instance of rtdimage object
    set image [[$w component image] get_image]

    # If no image loaded, return
    if { [$image isclear] } { return }

    # Get image dimensions (original size, not taking zoom factor into account)
    set image_height [$image height]
    set image_width [$image width]


    # Lower left corner of image (image coords -> screen coords)
    $image convert coords 0 0 image x1 y2 screen

    # Upper right corner of image (image coords -> screen coords)
    $image convert coords $image_width $image_height image x2 y1 screen

    # Get canvas (where the image is displayed in the RTD)
    set canvas [[$w component image] component canvas]

    # Get canvas dimensions (visible size on screen)
    set ch [winfo height $canvas]
    set cw [winfo width $canvas]

    # If image is larger than visible canvas, image corners = canvas corners
    if { $y1 < 0 } {
	set y1 0
    }

    if { $x1 < 0 } {
	set x1 0
    }

    if { $y2 > $ch } {
	set y2 $ch
    }

    if { $x2 > $cw } {
	set x2 $cw
    }

    # If cursor is out of image, do nothing (and return...)
    if { $x < $x1 || $x > $x2 || $y < $y1 || $y > $y2 } {
 	puts "out of image --> no action"
	return
    }

    # Get apparent image dimensions (includes the zoom factor)
    set dw [$image dispwidth]
    set dh [$image dispheight]
    
    if { $cw > $dw } {
	set scrollx [expr $cw/2.]
    } else {
 	set scrollx [expr $dw - $cw/2.]
    }

    if { $ch > $dh } {
	set scrolly [expr $ch/2.]
    } else {
 	set scrolly [expr $dh - $ch/2.]
    }


    $canvas config -scrollregion "[expr -$scrollx] [expr -$scrolly]  [expr $cw+$scrollx] [expr $ch+$scrolly]"

    # Convert cursor position to canvas coordinates
    $image convert coords $x $y screen xc yc canvas

    $image convert coords [expr $cw/2.] [expr $ch/2.] screen xcc ycc canvas

    set xFraction [expr ($xc-$cw/2.+$scrollx)/($cw+2.*$scrollx)]
    set yFraction [expr ($yc-$ch/2.+$scrolly)/($ch+2.*$scrolly)]


    if {$cw != 1 && $dw && $dh} {

	$image warp [expr int($xcc - $xc)] 0
	$canvas xview moveto $xFraction

	$image warp 0 [expr int($ycc - $yc)]
	$canvas yview moveto $yFraction
	
    }
    
}

proc fc_zoom_in { w } {

    set imageCtrl [$w component image]
    set image [$imageCtrl get_image]
    set canvas [$imageCtrl component canvas]

    set ch [winfo height $canvas]
    set cw [winfo width $canvas]

    $image convert coords [expr $cw/2.] [expr $ch/2.] screen xi yi image

    set scale [lindex [$image scale] 0]

    if { $scale == -2 } { set scale 0 }

    if { $scale < 20 } {
	incr scale
    }

    $imageCtrl scale $scale $scale

    $image convert coords $xi $yi image xc yc canvas

    # Get apparent image dimensions (includes the zoom factor)
    set dw [$image dispwidth]
    set dh [$image dispheight]

    if { $cw > $dw } {
	set scrollx [expr $cw/2.]
    } else {
 	set scrollx [expr $dw - $cw/2.]
    }

    if { $ch > $dh } {
	set scrolly [expr $ch/2.]
    } else {
 	set scrolly [expr $dh - $ch/2.]
    }

    $canvas config -scrollregion "[expr -$scrollx] [expr -$scrolly]  [expr $cw+$scrollx] [expr $ch+$scrolly]"

    set xFraction [expr ($xc-$cw/2.+$scrollx)/($cw+2.*$scrollx)]
    set yFraction [expr ($yc-$ch/2.+$scrolly)/($ch+2.*$scrolly)]

    if {$cw != 1 && $dw && $dh} {

	$canvas xview moveto $xFraction
	$canvas yview moveto $yFraction
	
    }
    
}

proc fc_zoom_out { w } {

    set imageCtrl [$w component image]
    set image [$imageCtrl get_image]
    set canvas [$imageCtrl component canvas]

    set ch [winfo height $canvas]
    set cw [winfo width $canvas]

    $image convert coords [expr $cw/2.] [expr $ch/2.] screen xi yi image

    set scale [lindex [$image scale] 0]

    if { $scale == 1 } { set scale -1 }

    if { $scale > -10 } {
	incr scale -1
    }

    $imageCtrl scale $scale $scale

    $imageCtrl scale $scale $scale

    $image convert coords $xi $yi image xc yc canvas

    # Get apparent image dimensions (includes the zoom factor)
    set dw [$image dispwidth]
    set dh [$image dispheight]
    
    if { $cw > $dw } {
	set scrollx [expr $cw/2.]
    } else {
 	set scrollx [expr $dw - $cw/2.]
    }

    if { $ch > $dh } {
	set scrolly [expr $ch/2.]
    } else {
 	set scrolly [expr $dh - $ch/2.]
    }


    $canvas config -scrollregion "[expr -$scrollx] [expr -$scrolly]  [expr $cw+$scrollx] [expr $ch+$scrolly]"

    set xFraction [expr ($xc-$cw/2.+$scrollx)/($cw+2.*$scrollx)]
    set yFraction [expr ($yc-$ch/2.+$scrolly)/($ch+2.*$scrolly)]


    if {$cw != 1 && $dw && $dh} {

	$canvas xview moveto $xFraction
	$canvas yview moveto $yFraction
	
    }

}

## Clear the canvas from all Finding Chart information
proc fc_clearFindingChart {w} {

    set canvas [[$w component image] component canvas]
    
    $canvas delete fcinfo
    update
    
}    

## Draw Finding Chart info in canvas
proc fc_drawFindingChart {w} {

    global fcPlugVar

    
    set image [[$w component image] get_image]
    set canvas [[$w component image] component canvas]

    if { [$image wcsheight] == "" } {

	tk_dialog .dia "WCS missing" "This image does not support\nthe WCS coordinate system !" \
	    warning 0 OK
	return
    }
    
    if { [$image isclear] } {
	tk_dialog .dia "No image" "No image is currently loaded !" \
	    warning 0 OK
	return
    }

    if { $fcPlugVar(runID) == "" } {
	tk_dialog .dia "Missing info" "You must provide\nthe run ID !" \
	    warning 0 OK
	return
    }

    if { $fcPlugVar(piName) == "" } {
	tk_dialog .dia "Missing info" "You must provide\nthe PI name !" \
	    warning 0 OK
	return
    }

    if { $fcPlugVar(obName) == "" } {
	tk_dialog .dia "Missing info" "You must provide\nthe OB name !" \
	    warning 0 OK
	return
    }

    if { $fcPlugVar(band) == "" } {
	tk_dialog .dia "Missing info" "You must provide the band \nor wavelength of the image !" \
	    warning 0 OK
	return
    }

    if { [catch {set fcPlugVar(color) [.fcDialog.infoColor get]} ] != 0 } {
	set fcPlugVar(color) black
    }

    set fcPlugVar(font) "-*-courier-bold-r-*-*-*-240-*-*-*-*-*-*"
    set fcPlugVar(font2) "-*-courier-bold-r-*-*-*-220-*-*-*-*-*-*"

    ## TEMP CODE

    .skycat1.image.panel.info.trans.rotate configure -state disabled
    .skycat1.image.panel.info.trans.flipx configure -state disabled
    .skycat1.image.panel.info.trans.flipy configure -state disabled

    $image rotate 0
    $image flip x 0
    $image flip y 0

    ## END TEMP CODE

    $image cmap file ramp.lasc
    $image itt file neg.iasc

    set image_height [$image height]
    set image_width [$image width]

    set fcPlugVar(pixscale) [lindex [$image wcsset] 2]
    set fcPlugVar(rotAngle) [lindex [$image wcsset] 7]

    set canvas_visible_height [winfo height $canvas]
    set canvas_visible_width [winfo width $canvas]


    ## Lower left corner of image (image coords -> screen coords)
    $image convert coords 0 0 image x1 y2 screen

    # Upper right corner of image (image coords -> screen coords)
    $image convert coords $image_width $image_height image x2 y1 screen

    if { [$image rotate] == 0 } {
	
	if { $y1 < 0 } {
	    set y1 0
	}

	if { $x1 < 0 } {
	    set x1 0
	}

	if { $y2 > $canvas_visible_height } {
	    set y2 $canvas_visible_height
	}

	if { $x2 > $canvas_visible_width } {
	    set x2 $canvas_visible_width
	}
	
    } else {

	if { $y1 > $canvas_visible_height } {
	    set y1 $canvas_visible_height
	}

	if { $x1 > $canvas_visible_width } {
	    set x1 $canvas_visible_width
	}

	if { $y2 < 0 } {
	    set y2 0
	}

	if { $x2 < 0 } {
	    set x2 0
	}

    }

    $canvas delete fcinfo
    update

    if { [$image rotate] == 0 } {
	$image convert coords $x1 $y1 screen \
	    fcPlugVar(x1_c) fcPlugVar(y1_c) canvas
	$image convert coords $x2 $y2 screen \
	    fcPlugVar(x2_c) fcPlugVar(y2_c) canvas
    } else {
	$image convert coords $x2 $y2 screen \
	    fcPlugVar(x1_c) fcPlugVar(y1_c) canvas
	$image convert coords $x1 $y1 screen \
	    fcPlugVar(x2_c) fcPlugVar(y2_c) canvas
    }


    ## Determine image scale and visible dimensions
    set fcPlugVar(scale) [lindex [$image scale] 0]

    if { $fcPlugVar(scale) < 0 } {
	set fcPlugVar(scale) [expr -1./$fcPlugVar(scale)]
    }

    set fcPlugVar(visImageHeight) \
	[expr $fcPlugVar(pixscale)*abs($y2-$y1)/(60*$fcPlugVar(scale))]

    set fcPlugVar(visImageWidth) \
	[expr $fcPlugVar(pixscale)*abs($x2-$x1)/(60*$fcPlugVar(scale))]


    ## Choice for location of FC info
    set fcLocation [.fcDialog.infoLocation get]

    switch $fcLocation {

	"top-left corner" {
	    set fcPlugVar(xInfo_c) [expr $fcPlugVar(x1_c) + 10]
	    set fcPlugVar(yInfo_c) [expr $fcPlugVar(y1_c) + 10]
	    set fcPlugVar(anchorInfo) "nw"
	    set fcPlugVar(origArrowX_c) "undef"
	    set fcPlugVar(origArrowY_c) "undef"
	    set fcPlugVar(origScaleX_c) [expr $fcPlugVar(x1_c) + 20]
	    set fcPlugVar(origScaleY_c) [expr $fcPlugVar(y2_c) - 20]

	    set fcPlugVar(userChosenFlag) false
	    fc_plotRunInfo $w
	    fc_plotScale $w
	    fc_plotArrow $w br
	}

	"top-right corner" {
	    set fcPlugVar(xInfo_c) [expr $fcPlugVar(x2_c) - 10]
	    set fcPlugVar(yInfo_c) [expr $fcPlugVar(y1_c) + 10]
	    set fcPlugVar(anchorInfo) "ne"
	    set fcPlugVar(origArrowX_c) "undef"
	    set fcPlugVar(origArrowY_c) "undef"
	    set fcPlugVar(origScaleX_c) [expr $fcPlugVar(x1_c) + 20]
	    set fcPlugVar(origScaleY_c) [expr $fcPlugVar(y2_c) - 20]

	    set fcPlugVar(userChosenFlag) false
	    fc_plotRunInfo $w
	    fc_plotScale $w
	    fc_plotArrow $w br
	}

	"bottom-left corner" {
	    set fcPlugVar(xInfo_c) [expr $fcPlugVar(x1_c) + 10]
	    set fcPlugVar(yInfo_c) [expr $fcPlugVar(y2_c) - 10]
	    set fcPlugVar(anchorInfo) "nw"
	    set fcPlugVar(origArrowX_c) "undef"
	    set fcPlugVar(origArrowY_c) "undef"
	    set fcPlugVar(origScaleX_c) [expr $fcPlugVar(x1_c) + 20]
	    set fcPlugVar(origScaleY_c) [expr $fcPlugVar(y1_c) + 20]

	    set fcPlugVar(userChosenFlag) false
	    fc_plotRunInfo $w
	    fc_plotScale $w
	    fc_plotArrow $w br
	}

	"bottom-right corner" {
	    set fcPlugVar(xInfo_c) [expr $fcPlugVar(x2_c) - 10]
	    set fcPlugVar(yInfo_c) [expr $fcPlugVar(y2_c) - 10]
	    set fcPlugVar(anchorInfo) "ne"
	    set fcPlugVar(origArrowX_c) "undef"
	    set fcPlugVar(origArrowY_c) "undef"
	    set fcPlugVar(origScaleX_c) [expr $fcPlugVar(x1_c) + 20]
	    set fcPlugVar(origScaleY_c) [expr $fcPlugVar(y1_c) + 20]

	    set fcPlugVar(userChosenFlag) false
	    fc_plotRunInfo $w
	    fc_plotScale $w
	    fc_plotArrow $w bl
	}

	"user chosen" {

	    set fcPlugVar(anchorInfo) "nw"

	    if { $fcPlugVar(userChosenFlag) == "true" } {

		fc_plotRunInfo $w
		fc_plotScale $w
		fc_plotArrow $w
	    
	    } else {
		
		tk_dialog .dia "Choose anchor point" \
		    "Click 3 times in canvas to choose where to put:\n- program-related info\n- direction arrows\n- image scale" \
		    info 0 OK

		$canvas config -cursor cross

		bind $canvas <Button-1> {
		    set ::fcXCoord %x
		    set ::fcYCoord %y
		}

		vwait ::fcXCoord
		
		$canvas config -cursor left_ptr

		$image convert coords $::fcXCoord $::fcYCoord screen \
		    fcPlugVar(xInfo_c) fcPlugVar(yInfo_c) canvas

		fc_plotRunInfo $w

		$canvas config -cursor cross

		vwait ::fcXCoord

		$canvas config -cursor left_ptr
		
		$image convert coords $::fcXCoord $::fcYCoord screen \
		    fcPlugVar(origArrowX_c) fcPlugVar(origArrowY_c) canvas

		fc_plotArrow $w

		$canvas config -cursor cross

		vwait ::fcXCoord

		$canvas config -cursor left_ptr

		$image convert coords $::fcXCoord $::fcYCoord screen \
		    fcPlugVar(origScaleX_c) fcPlugVar(origScaleY_c) canvas

		fc_plotScale $w

		set fcPlugVar(userChosenFlag) true

		bind $canvas <Button-1> {}
	    }
	}
    }

    set fcPlugVar(draw) "true"

}

# Save the finding chart to a JPEG file
proc fc_saveFindingChart { w } {

    global fcPlugVar env
    
    if { $fcPlugVar(draw) == "false" } {
        tk_dialog .fcdia "Missing info" \
            "You have to draw the Finding Chart\nbefore saving it !" \
            error 0 OK
        return
    }

    if { [info exists env(FC_DIR)] == 1 } {
	set fcpath $env(FC_DIR)
    } else {
	set fcpath ""
	catch {set fcpath $fcPlugVar(startDir)}
    }

    regsub -all {[()]} $fcPlugVar(runID) "" runId

    set fcList [glob -nocomplain "$fcpath/$runId.*.jpg"]

    if { [llength $fcList] == 0 } {
        set index 1
    } else {
        set lastFile [file rootname [file tail [lindex $fcList end]]]
        if { [regexp {[0-9][0-9][0-9]$} $lastFile index] != 1 } {
            set index 1
        } else {
	    set index [string trimleft $index "0"]
	    set index [expr $index + 1]
	}
    }

    set index [format "%03d" $index]
    set filename "$runId.$index.jpg"

    set types { { {JPEG files} {.jpg .jpeg} } }

    set file [tk_getSaveFile -title "Save Finding Chart" \
		  -filetypes $types \
		  -parent $w \
		  -initialdir $fcpath \
		  -initialfile $filename \
		  -defaultextension "jpg" ]

    if { $file == "" } {
	return
    }

    destroy $w.__tk_filedialog

    set fcPlugVar(startDir) [file dirname $file]

    # Get canvas (where the image is displayed in the RTD)
    set imageCtrl [$w component image]
    set image [$imageCtrl get_image]
    set canvas [$imageCtrl component canvas]
    
    # Get image dimensions (original size, not taking zoom factor into account)
    set image_height [$image height]
    set image_width [$image width]

    # Get canvas dimensions (visible size on screen)
    set ch [winfo height $canvas]
    set cw [winfo width $canvas]

    # Get coordinates of corners of image in screen coordinates
    $image convert coords 1 1 image im_tl_x im_br_y screen
    $image convert coords $image_width $image_height image \
	im_br_x im_tl_y screen


    if { $im_tl_x < 0 } { 
	set top_left_corner_x 0
    } else {
	set top_left_corner_x $im_tl_x
    }

    if { $im_br_x > $cw } {
	set bottom_right_corner_x $cw
    } else {
	set bottom_right_corner_x $im_br_x
    }

    if { $im_tl_y < 0 } {
	set top_left_corner_y 0
    } else {
	set top_left_corner_y $im_tl_y
    }

    if { $im_br_y > $ch } {
	set bottom_right_corner_y $ch
    } else {
	set bottom_right_corner_y $im_br_y
    }


    set viewable_image_width \
	[expr int($bottom_right_corner_x - $top_left_corner_x)]
    set viewable_image_height \
	[expr int($bottom_right_corner_y - $top_left_corner_y)]
    
    # Get corners of viewable image (screen --> image)
    $image convert coords \
	$top_left_corner_x $bottom_right_corner_y screen llx lly image
    $image convert coords \
	$bottom_right_corner_x $top_left_corner_y screen urx ury image


    if { $fcPlugVar(jpegResolution) == "normal" } {

	raise $w
	update

	set retVal [catch {image create photo -format window -data $canvas} ph]
	
	if { $retVal != 0 } {
	    tk_dialog .fcdia "Cannot create image" \
		"ERROR: Cannot create image from canvas window" \
		error 0 OK
	    return
	}
	
	image create photo fc_print -height $viewable_image_height -width $viewable_image_width

	fc_print copy $ph -from \
	    [expr int($top_left_corner_x)] \
	    [expr int($top_left_corner_y)] \
	    [expr int($bottom_right_corner_x)] \
	    [expr int($bottom_right_corner_y)]

	set retVal [ catch {fc_print write -format jpeg $file} ]
	
	image delete fc_print
	image delete $ph

	if { $retVal != 0 } {
	    tk_dialog .fcdia "Cannot write JPEG file" \
		"ERROR: Cannot save Finding Chart to JPEG file" \
		error 0 OK
	    return
	}

    } else {


	# Required final size of finding chart
	# (dimensions must both reach the following minimums)
	set required_fc_height 1200
	set required_fc_width 1200

	if { [expr $viewable_image_height < $required_fc_height] || \
		 [expr $viewable_image_width < $required_fc_width] } {

	    set mag_ok false
	    
	} else {

	    set mag_ok true

	}

	set mag 0

	while { $mag_ok == "false" } {

	    fc_zoom_in $w
	    incr mag
	    $image convert coords $llx $lly image x1 y2 canvas
	    $image convert coords $urx $ury image x2 y1 canvas

	    set zoom_height [expr $y2 - $y1]
	    set zoom_width [expr $x2 - $x1]

	    if { $zoom_height >= $required_fc_height && \
		     $zoom_width >= $required_fc_width } {
		set mag_ok true
	    }
	}

	update

	# Final size of finding chart
	set fc_height [expr int($zoom_height)]
	set fc_width [expr int($zoom_width)]

	set loop_x [expr int(ceil($zoom_width/$viewable_image_width))]
	set loop_y [expr int(ceil($zoom_height/$viewable_image_height))]

	lassign [$canvas cget -scrollregion] c_left c_top c_right c_bottom

	set x_topleft [expr ($x1-$c_left)]
	set y_topleft [expr ($y1-$c_top)]

	# Create empty photo image
	image create photo fc_image -height $fc_height -width $fc_width

	raise $w

	for {set i 0} {$i < $loop_x} {incr i} {

	    set x [expr $x_topleft + $i * $viewable_image_width]
	    set xfraction [expr $x/($c_right-$c_left)]

	    $canvas xview moveto $xfraction

	    for {set j 0} {$j < $loop_y} {incr j} {

		set y [expr $y_topleft + $j * $viewable_image_height]
		set yfraction [expr $y/($c_bottom-$c_top)]

		$canvas yview moveto $yfraction
		
		update
		
		image create photo canvas_grab -format window -data $canvas

		fc_image copy canvas_grab -to \
		    [expr int($i * $viewable_image_width)] \
		    [expr int($j * $viewable_image_height)] \
		    [expr int(($i+1)*$viewable_image_width)] \
		    [expr int(($j+1)*$viewable_image_height)]

		image delete canvas_grab
	    }
	}

	fc_image write -format "jpeg -quality 30" $file

	
	# Cleaning up...
	image delete fc_image

	# Restoring previous state...
	for {set i 1} { $i <= $mag } {incr i} {
	    fc_zoom_out $w
	}

	$image convert coords $llx $ury image x1 y1 canvas

	lassign [$canvas cget -scrollregion] c_left c_top c_right c_bottom

	set xfraction [expr ($x1-$c_left)/($c_right-$c_left)]
	set yfraction [expr ($y1-$c_top)/($c_bottom-$c_top)]

	$canvas xview moveto $xfraction
	$canvas yview moveto $yfraction

	return


    }
}


## Save the Finding Chart to a Postscript File
proc fc_printFindingChart { w } {

    global fcPlugVar env
    
    if { $fcPlugVar(draw) == "false" } {
        tk_dialog .fcdia "Missing info" \
            "You have to draw the Finding Chart\nbefore printing it !" \
            error 0 OK
        return
    }
    
#    fc_drawFindingChart $w 

    set canvas [[$w component image] component canvas]
    set image [[$w component image] get_image]

    set object [$image object]
    set file [file tail [$w cget -file] ]
    set center [$image wcscenter]
    set user [id user]
    set app [lindex [winfo name .] 0]
    set date [clock format [clock seconds] -format {%b %d, %Y at %H:%M:%S}]

    if { [winfo exists $w.fcPrint] == 1 } {

 	wm deiconify $w.fcPrint
	$w.fcPrint configure \
	    -image [$w component image] \
	    -show_footer 1 \
	    -transient 1 \
	    -whole_canvas 1 \
	    -top_left "ESO\n$object" \
	    -top_right "$file\n$center" \
	    -bot_left "$user/$app" \
	    -bot_right "$date"

	update

    } else {

	RtdImagePrint $w.fcPrint \
	    -image [$w component image] \
	    -show_footer 1 \
	    -transient 1 \
	    -whole_canvas 1 \
	    -top_left "ESO\n$object" \
	    -top_right "$file\n$center" \
	    -bot_left "$user/$app" \
	    -bot_right "$date"

	update
    }

    if { [info exists env(FC_DIR)] == 1 } {

	set fcpath $env(FC_DIR)

    } else {

	set filename ""
 	catch {set filename [$w.fcPrint.print_to_file get]}

	set fcpath [file dirname $filename]
	if { $fcpath == "." } {
	    set fcpath $fcPlugVar(startDir)
	}
    }

    regsub -all {[()]} $fcPlugVar(runID) "" runId
    
    set fcList [glob -nocomplain "$fcpath/$runId.*.ps"]

    if { [llength $fcList] == 0 } {
        set index 1
    } else {
        set lastFile [file rootname [file tail [lindex $fcList end]]]
        if { [regexp {[0-9][0-9][0-9]$} $lastFile index] != 1 } {
            set index 1
        } else {
	    set index [string trimleft $index "0"]
	    set index [expr $index + 1]
	}
    }
    
    set index [format "%03d" $index]

    set filename "$fcpath/$runId.$index.ps"

    $w.fcPrint.print_to_file configure -value $filename
    $w.fcPrint.print_to_file configure -value ""

}

proc fc_plotRunInfo { w } {

    global fcPlugVar

    set canvas [[$w component image] component canvas]
    set draw [[$w component image] component draw]

    # Writing run info on finding chart
    set runId $fcPlugVar(runID)
    set PIName $fcPlugVar(piName)
    set targetName $fcPlugVar(obName)
    set band $fcPlugVar(band)

    # Put runId, PI name, OB name and band on canvas

    $canvas create text $fcPlugVar(xInfo_c) $fcPlugVar(yInfo_c) \
	-text "$runId" \
	-fill $fcPlugVar(color) \
	-font $fcPlugVar(font) \
	-tags {fcinfo runID userInfo} \
	-justify left -anchor $fcPlugVar(anchorInfo)

    lassign [$canvas bbox runID] tlx tly lrx lry
    
    $canvas create text $fcPlugVar(xInfo_c) $lry \
	-text "$PIName" \
	-fill $fcPlugVar(color) \
	-font $fcPlugVar(font) \
	-tags {fcinfo PIName userInfo} \
	-justify left -anchor $fcPlugVar(anchorInfo)

    lassign [$canvas bbox PIName] tlx tly lrx lry

    $canvas create text $fcPlugVar(xInfo_c) $lry \
	-text "$targetName" \
	-fill $fcPlugVar(color) \
	-font $fcPlugVar(font) \
	-tags {fcinfo userInfo} \
	-justify left -anchor $fcPlugVar(anchorInfo)

    lassign [$canvas bbox userInfo] tlx tly lrx lry

    $canvas create text $fcPlugVar(xInfo_c) $lry \
	-text "band: $band" \
	-fill $fcPlugVar(color) \
	-font $fcPlugVar(font) \
	-tags {fcinfo userInfo} \
	-justify left -anchor $fcPlugVar(anchorInfo)

    lassign [$canvas bbox userInfo] tlx tly lrx lry

    if { $lrx > $fcPlugVar(x2_c) } {
	$canvas move userInfo [expr $fcPlugVar(xInfo_c) - $lrx] 0
    } 

    if { $lry > $fcPlugVar(y2_c) } {
	$canvas move userInfo 0 [expr $fcPlugVar(yInfo_c) - $lry] 
    }

    if { $tly < $fcPlugVar(y1_c) } {
	$canvas move userInfo 0 [expr $fcPlugVar(yInfo_c) - $tly]
    }

    if { $tlx < $fcPlugVar(x1_c) } {
	$canvas move userInfo [expr $fcPlugVar(xInfo_c) - $tlx] 0
    }

    $draw add_object_bindings userInfo

    $canvas bind userInfo <ButtonRelease-1> \
 	"+ [code eval $draw deselect_objects]"

}

proc fc_plotArrow { w {anchor ""}} {

    global fcPlugVar

    set image [[$w component image] get_image]
    set canvas [[$w component image] component canvas]
    set draw [[$w component image] component draw]

    # Get size of visible image in arcseconds
    set vis_height_arcsec [expr $fcPlugVar(visImageHeight) * 60.]
    set vis_width_arcsec [expr $fcPlugVar(visImageWidth) * 60.]

    # Set size of compass to percentage of image size
    set compass_size [expr [min $vis_height_arcsec $vis_width_arcsec] * .2]
    set compass_size_deg [expr $compass_size / 3600.]

    # Get equinox from image
    set equinox [$image wcsequinox]
    set deg_eq "deg $equinox"

    if { $fcPlugVar(origArrowX_c) == "undef" } {
	set xc [expr ($fcPlugVar(x2_c) + $fcPlugVar(x1_c))/2.]
	set yc [expr ($fcPlugVar(y2_c) + $fcPlugVar(y1_c))/2.]
    } else {
	set xc $fcPlugVar(origArrowX_c)
	set yc $fcPlugVar(origArrowY_c)
    }

    # Get coords of compass origin in ra/dec system
    $image convert coords $xc $yc canvas ra0 dec0 $deg_eq	

    # Compute end points of compass in ra/dec system
    set ra1 [expr $ra0 + $compass_size_deg/cos(($dec0/180.) * 3.14159)]
    if { $ra1 < 0 } {
	set ra1 [expr 360 + $ra1]
    }

    set dec1 [expr $dec0 + $compass_size_deg]
    if { $dec1 >= 90 } {
	set dec1 [expr 180 - $dec1]
    }

    # Get canvas coords of end points
    set retVal [catch {$image convert coords $ra1 $dec0 $deg_eq arrowEastX_c arrowEastY_c canvas}]

    if { $retVal != 0 } {
	tk_dialog .fcdia "Cannot draw direction arrows" \
	    "ERROR: cannot draw direction arrows. You are trying to draw\nthe arrows outside of the image" \
	    error 0 OK
	set fcPlugVar(userChosenFlag) "false"
	return
    }

    $image convert coords $ra0 $dec1 $deg_eq arrowNorthX_c arrowNorthY_c canvas

    # Draw compass line on canvas
    $canvas create line $arrowEastX_c $arrowEastY_c \
	$xc $yc \
	$arrowNorthX_c $arrowNorthY_c \
	-fill $fcPlugVar(color) -width 2 -arrow both \
	-tags {fcinfo direction}

    set northX_c [expr $arrowNorthX_c + 15 * ($arrowEastX_c - $xc)/abs($arrowEastX_c - $xc)]
    set northY_c $arrowNorthY_c

    set eastX_c $arrowEastX_c
    set eastY_c [expr $arrowEastY_c + 15 * ($arrowNorthY_c - $yc)/abs($arrowNorthY_c - $yc)]

    $canvas create text $northX_c $northY_c \
	-text "N" -fill $fcPlugVar(color) -font $fcPlugVar(font2) \
	-tags {fcinfo direction}

    $canvas create text $eastX_c $eastY_c \
	-text "E" -fill $fcPlugVar(color) -font $fcPlugVar(font2) \
	-tags {fcinfo direction}

    lassign [$canvas bbox direction] tlx tly lrx lry

    if { $anchor == "br" } {
	$canvas move direction [expr $fcPlugVar(x2_c) - 15 - $lrx] [expr $fcPlugVar(y2_c) - 15 - $lry]
    }

    if { $anchor == "bl" } {
	$canvas move direction [expr $fcPlugVar(x1_c) + 15 - $tlx] [expr $fcPlugVar(y2_c) - 15 - $lry]
    }

    if { $anchor == "tl" } {
	$canvas move direction [expr $fcPlugVar(x1_c) + 15 - $tlx] [expr $fcPlugVar(y1_c) + 15 - $lry]
    }

    if { $anchor == "tr" } {
	$canvas move direction [expr $fcPlugVar(x2_c) - 15 - $tlx] [expr $fcPlugVar(y1_c) + 15 - $lry]
    }

    $draw add_object_bindings direction


    $canvas bind direction <ButtonRelease-1> \
 	"+ [code eval $draw deselect_objects]"

}

proc fc_plotScale { w } {

    global fcPlugVar

    set image [[$w component image] get_image]
    set canvas [[$w component image] component canvas]
    set draw [[$w component image] component draw]

    ### Scale

    if { $fcPlugVar(visImageHeight) > $fcPlugVar(visImageWidth) } {

	set scaleSize_s [expr floor(60.*$fcPlugVar(visImageHeight)/4.)]

    } else {

	set scaleSize_s [expr floor(60.*$fcPlugVar(visImageWidth)/4.)]

    }    

    set scaleList { {1 "1\""} {5 "5\""} {10 "10\""} \
			{20 "20\""} {30 "30\""} {40 "40\""} \
			{60 "1'"} {120 "2'"} }
    set dmin 1000

    foreach sc $scaleList {

	set scVal [lindex $sc 0]
	set d [expr abs($scVal-$scaleSize_s)]

	if { $d < $dmin } {
	    set dmin $d
	    set chosenScale $scVal
	    set chosenLegend [lindex $sc 1]
	}
	
    }

    set lengthScale \
	[expr $chosenScale / $fcPlugVar(pixscale) * $fcPlugVar(scale)]
    
    set endScaleX_c $fcPlugVar(origScaleX_c)
    set endScaleY_c [expr $fcPlugVar(origScaleY_c) - $lengthScale]
    set origScaleLegendX_c [expr $fcPlugVar(origScaleX_c) + 10]
    set origScaleLegendY_c [expr $fcPlugVar(origScaleY_c) - $lengthScale/2]
    
    $canvas create line $fcPlugVar(origScaleX_c) $fcPlugVar(origScaleY_c) \
	$endScaleX_c $endScaleY_c \
	-width 3 -fill $fcPlugVar(color) -tags {fcinfo scale}  

    $canvas create line [expr $fcPlugVar(origScaleX_c) - 5] \
	$fcPlugVar(origScaleY_c) \
	[expr $fcPlugVar(origScaleX_c) + 5] \
	$fcPlugVar(origScaleY_c) \
	-width 3 -fill $fcPlugVar(color) -tags {fcinfo scale}  

    $canvas create line [expr $endScaleX_c - 5] \
	$endScaleY_c \
	[expr $endScaleX_c + 5] \
	$endScaleY_c \
	-width 3 -fill $fcPlugVar(color) -tags {fcinfo scale}  

    $canvas create text $origScaleLegendX_c $origScaleLegendY_c \
	-text $chosenLegend -fill $fcPlugVar(color) -font $fcPlugVar(font2) \
	-tags {fcinfo scale} -anchor w

    lassign [$canvas bbox scale] tlx tly lrx lry

    if { $lrx > $fcPlugVar(x2_c) } {
	$canvas move scale [expr $fcPlugVar(origScaleX_c) - $lrx] 0
    } 

    if { $lry > $fcPlugVar(y2_c) } {
	$canvas move scale 0 [expr $fcPlugVar(origScaleY_c) - $lry] 
    }

    if { $tly < $fcPlugVar(y1_c) } {
	$canvas move scale 0 [expr $fcPlugVar(origScaleY_c) - $tly]
    }

    if { $tlx < $fcPlugVar(x1_c) } {
	$canvas move scale [expr $fcPlugVar(origScaleX_c) - $tlx] 0
    }

    $draw add_object_bindings scale

    $canvas bind scale <ButtonRelease-1> \
 	"+ [code eval $draw deselect_objects]"
}
